function ZhuMenuElementsController(ScreenService, templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService, restEngine, ZHPromise){
	var MENU = "M";
	var GROUP_MENU = "G";
	var IGNORED_MENUS = ['zhHistory'];
	var idErrorMessages = {"errorMenus": "", "errorContainers": ""};

	var getWidgetElements = function(){
		return templateManager.container.getWidget(ZhuAclConfig.menuElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = {"errorMenus": "", "errorContainers": ""};
	};

	var showIdErrorsMessage = function(){
		var messageCode;
		var errorMenus = idErrorMessages.errorMenus;
		var errorContainers = idErrorMessages.errorContainers;
		var messageParameters = {
			elementsIds: errorMenus,
			elementsWithoutContainer: errorContainers
		};

		if(errorMenus && errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG;
		} else if(errorMenus){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID;
		} else if(errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_CONTAINER_CONFIG;
		}

		if(messageCode){
			MessageService.showMessageByCode(messageCode, messageParameters, { type: 'alert' });
		}
	};

	var addErrorMenuId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorMenus += "<br> " + nameToShow + ";";
	};

	var addErrorContainerId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorContainers += "<br> " + nameToShow + ";";
	};

	this.elementsActionEventEdit = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			var messageConfig = {
				type: 'menu',
				subType: 'submenus'
			};
			currentRow.isEditable = !currentRow.isEditable;
			ZhuAclCRUDController.changeIsEditable(elementsWidget, currentRow, messageConfig);
		}
	};

	this.elementsActionEventView = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			var messageConfig = {
				type: 'menu',
				subType: 'submenus'
			};
			currentRow.isVisible = !currentRow.isVisible;
			currentRow.isEditable = currentRow.isVisible;
			ZhuAclCRUDController.changeIsVisible(elementsWidget, currentRow, messageConfig);
		}
	};

	this.elementsActionEventBack = function(){
		var elementsWidgetParent = getWidgetElements().parent;
		return ScreenService.closeSwipe().then(function(){
			elementsWidgetParent.list();
			elementsWidgetParent.activate();
		});
	};

	var addContainerProperties = function(item, menuItem, rules){
		var containerId = menuItem.containerId;
		var containerLabel = menuItem.containerLabel || menuItem.containerName;
		if(containerId && containerLabel){
			var containerRule = ZhuAclController.getElementRule(rules, containerId);
			item.CONTAINER_ID = containerId;
			item.CONTAINER_NAME = menuItem.containerName || ZhuAclController.prepareLabelToName(containerLabel);
			item.CONTAINER_LABEL = containerLabel;
			item.CONTAINER_RULE_ID = containerRule.RULE_ID;
		} else if(menuItem.windowName && !menuItem.buildedMenu){
			addErrorContainerId(menuItem);
		}
	};

	var planningMenuItem = function(menuItem, parentMenuItem, rules){
		var item;
		var uniqueId = menuItem.id;
		if(uniqueId){
			var parentId = parentMenuItem && parentMenuItem.id;
			var itemType = !parentId ? GROUP_MENU : MENU;
			item = ZhuAclController.getPlannedItemToInsertInDataSet(rules, menuItem, itemType, parentId);
			item.buildedMenu = menuItem.buildedMenu;
			addContainerProperties(item, menuItem, rules);
		} else {
			addErrorMenuId(menuItem);
		}

		return item;
	};

	var addMenusInDataSource = function(widget, rules, menus, parentMenu){
		menus.forEach(function(menu, index){
			var item = planningMenuItem(menu, parentMenu, rules);
			if(item){
				widget.dataSource.data.push(item);
				if (menu.menus && menu.menus.length > 0){
					addMenusInDataSource(widget, rules, menu.menus, menu);
				}
			}
		});
	};

	var getProjectMenu = function(){
		var projectMenu = Util.clone(templateManager.project.groupMenu);
		return projectMenu.filter(function(menu){
			return IGNORED_MENUS.indexOf(menu.name) == -1;
		});
	};

	var extraRulesPromise;

	var loadExtraRules = function(extraElementsPath){
		if(!extraRulesPromise){
			if(extraElementsPath){
				extraRulesPromise = restEngine.requestMetaData(Util.getRightPath(extraElementsPath), {});
			} else {
				extraRulesPromise = ZHPromise.when();
			}
		}
	};

	var addExtraElements = function(widget, projectRules, defaultRule){
		var getRule = function(id){
			return (projectRules[id] && projectRules[id].ruleRow) || {};
		};

		var getVisibility = function(rule){
			return (rule || defaultRule) !== 'I';
		};

		extraRulesPromise.then(function(response){
			if(response && response.data){
				response.data.forEach(function(row){
					var rule = getRule(row.UNIQUE_ID);
					row.RULE = rule.RULE;
					row.RULE_ID = rule.ID;
					row.isVisible = getVisibility(rule.RULE);
					row.extraElement = true;
				});
				widget.dataSource.data = widget.dataSource.data.concat(response.data);
			}
		});
	};

	this.loadPlanningTree = function(){
		var elementsWidget = getWidgetElements();
		var userId = templateManager.userId;
		var defautRule = templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule();
		var rules = ZhuAclController.getRulesFromDatasource(elementsWidget.dataSource.data);
		var newMenu = getProjectMenu();

		clearIdErrorsMessage();
		loadExtraRules(elementsWidget.extraElementsPath);
		aclEngine.processMenuAcl(newMenu, rules, templateManager.containers, userId, defautRule);
		elementsWidget.dataSource.data = [];
		addMenusInDataSource(elementsWidget, rules, newMenu);
		addExtraElements(elementsWidget, rules, defautRule);
		showIdErrorsMessage();
	};

	this.reloadElementsWidget = function(){
		getWidgetElements().reload();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuMenuElementsController', ZhuMenuElementsController);
});